# coding : UTF-8
import sys
sys.path.append("../../tests")
import towhee
import time
import threading
import numpy as np
import random
import operator
from towhee import pipeline
from common import common_func as cf
from towhee.functional import DataCollection

class TestDataCollectionAPIsInvalid:
    """ Test case of invalid data collection interface """

    def test_data_collection_API_no_parameter(self, API_name):
        """
        target: test data collection APIs for invalid scenario
        method:  call APIs with no parameters
        expected: raise exception
        """
        API = eval("DataCollection.%s" % API_name)
        try:
            dc = API()
        except Exception as e:
            assert "missing" in str(e)

        return True

    def test_data_collection_API_not_supported_type(self, API_name):
        """
        target: test data collection APIs for invalid scenario
        method: call APIs with not supported data types
        expected: raise exception
        """
        if API_name in ["filter", "batch", "rolling", "pmap", "map", "mmap"]:
            return True
        not_support_datas = ["string", {1, "s", 2, 3}]
        API = eval("DataCollection.%s" % API_name)
        for not_support_data in not_support_datas:
            try:
                dc = API(not_support_data)
            except Exception as e:
                assert "no attribute" in str(e)

        return True

class TestDataCollectionAPIsValid:
    """ Test case of invalid data collection interface """
    
    def test_data_collection_map_lambda(self):
        """
        target: test filter() API for DataCollection 
        method: create a data collection and map data with lambda
        expected: return map successfully
        """
        size = 5
        dc = DataCollection.range(size)
        dc = dc.map(lambda x: x+1)
        result = dc.to_list()
        assert len(result) == size

        return True

    def test_data_collection_map_inter(self):
        """
        target: test map() API for DataCollection 
        method: create an iter data collection and map it
        expected: return map successfully
        """
        size = 5
        data = iter(range(size))
        dc = towhee.dc(data)
        dc = dc.map(str)
        result = dc.to_list()
        assert len(result) == size
        
        return True
     
    def test_data_collection_map_empty(self):
        """
        target: test map() API for DataCollection 
        method: create an empty data collection and map it
        expected: return map successfully
        """
        data = []
        dc =towhee.dc(data)
        dc = dc.map(str)
        result = dc.to_list()
        assert result == data
        
        return True

    def test_data_collection_map_large_size(self):
        """
        target: test map() API for DataCollection 
        method: create a large size data collection and map it
        expected: return map successfully
        """
        data = 10000000
        dc = DataCollection.range(data)
        dc = dc.map(str)
        result = dc.to_list()
        assert len(result) == data
        
        return True
    
    def test_data_collection_filter_lambda(self):
        """
        target: test filter() API for DataCollection 
        method: create a data collection and filter data with lambda
        expected: return filter successfully
        """
        size = 5
        dc = DataCollection.range(size)
        dc = dc.filter(lambda x: x+1)
        result = dc.to_list()
        assert len(result) == size

        return True

    def test_data_collection_filter_inter(self):
        """
        target: test filter() API for DataCollection 
        method: create an iter data collection and filter it
        expected: return filter successfully
        """
        size = 5
        data = iter(range(size))
        dc = towhee.dc(data)
        dc = dc.filter(str)
        result = dc.to_list()
        assert len(result) == size
        
        return True
     
    def test_data_collection_filter_empty(self):
        """
        target: test filter() API for DataCollection 
        method: create an empty data collection and filter it
        expected: return filter successfully
        """
        data = []
        dc = towhee.dc(data)
        dc = dc.filter(str)
        result = dc.to_list()
        assert result == data

        return True

    def test_data_collection_filter_large_size(self):
        """
        target: test filter() API for DataCollection 
        method: create a large size data collection and filter it
        expected: return filter successfully
        """
        data = 10000000
        dc = DataCollection.range(data)
        dc = dc.filter(str)
        result = dc.to_list()
        assert len(result) == data

        return True
    
    def test_data_collection_zip_lambda(self):
        """
        target: test zip() API for DataCollection 
        method: create a data collection and zip  with lambda
        expected: return zip successfully
        """
        size = 5
        dc1 = DataCollection.range(size)
        dc2 = dc1.map(lambda x: x+1)
        dc3 = dc1.zip(dc2)
        result = dc3.to_list()
        assert len(result) == size
        return True

    def test_data_collection_zip_inter(self):
        """
        target: test zip() API for DataCollection 
        method: create an iter data collection and zip it
        expected: return zip successfully
        """
        size = 7
        data = iter(range(size))
        dc1 = towhee.dc(data)
        dc2 = dc1.map(str)
        dc3 = dc1.zip(dc2)
        result = dc3.to_list()
        assert len(result) == int(size/2)
        
        return True
     
    def test_data_collection_zip_empty(self):
        """
        target: test zip() API for DataCollection 
        method: create an empty data collection and zip it
        expected: return zip successfully
        """
        data = []
        dc1 = towhee.dc(data)
        size = 1
        dc2 = DataCollection.range(size)
        dc3 = dc1.zip(dc2)
        result = dc3.to_list()
        assert result == data

        return True

    def test_data_collection_zip_large_size(self):
        """
        target: test zip() API for DataCollection 
        method: create a large size data collection and zip it
        expected: return zip successfully
        """
        data = 10000000
        dc1 = DataCollection.range(data)
        dc2 = dc1.filter(str)
        dc3 = dc1.zip(dc2)
        result = dc3.to_list()
        assert len(result) == data

        return True

    def test_data_collection_batch_data(self):
        """
        target: test batch() API for DataCollection 
        method: create a data collection and batch it
        expected: return batch successfully
        """
        data = 10
        dc = DataCollection.range(data)
        size = 3
        result = [list(batch) for batch in dc.batch(size, drop_tail=True)]
        assert len(result) == int(data/size) 

        return True

    def test_data_collection_batch_inter(self):
        """
        target: test batch() API for DataCollection 
        method: create an iter data collection and batch it
        expected: return batch successfully
        """
        data_size = 6
        data = iter(range(data_size))
        dc = towhee.dc(data)
        size = 3
        result = [list(batch) for batch in dc.batch(size, drop_tail=True)]
        assert len(result) == int(data_size/size) 

        return True

    def test_data_collection_batch_large_size(self):
        """
        target: test batch() API for DataCollection 
        method: create a large size data collection and batch with large size
        expected: return batch successfully
        """
        data = 10000000
        dc = DataCollection.range(data)
        size = 1000000
        result = [list(batch) for batch in dc.batch(size, drop_tail=True)]
        assert len(result) == int(data/size)   

        return True

    def test_data_collection_batch_size_empty(self):
        """
        target: test batch() API for DataCollection 
        method: create a data collection and batch size is empty
        expected: return batch successfully
        """
        data = 5
        dc = DataCollection.range(data)
        size = []
        result = [list(batch) for batch in dc.batch(size)]
        assert len(result) == 1

        return True
    
    
    def test_data_collection_rolling_drop_head(self):
        """ 
        target: test rolling() API for DataCollection 
        method: create a data collection and rolling it with drop_head is False
        expected: return rolling successfully
        """
        data = 5
        dc = DataCollection.range(data)
        size = 3
        result = [list(batch) for batch in dc.rolling(size, drop_head=False)]
        assert len(result) == data

        return True
    
    def test_data_collection_rolling_drop_tail(self):
        """ 
        target: test rolling() API for DataCollection 
        method: create a data collection and rolling it with drop_tail is False
        expected: return rolling successfully
        """
        data = 10
        dc = DataCollection.range(data)
        size = 3
        result = [list(batch) for batch in dc.rolling(size, drop_tail=False)]
        assert len(result) == data

        return True

    def test_data_collection_rolling_large_size(self):
        """    
        target: test rolling() API for DataCollection 
        method: create a large size data collection and rolling it with large size
        expected: return rolling successfully
        """
        data = 100000
        dc = DataCollection.range(data)
        size = 1000
        result = [list(batch) for batch in dc.rolling(size, drop_tail=False)]
        assert len(result) == data

        return True

    def test_data_collection_rolling_size_empty(self):
        """
        target: test rolling() API for DataCollection 
        method: create a data collection and rolling size is empty
        expected: return rolling successfully
        """
        data = 10
        dc = DataCollection.range(data)
        size = []
        result = [list(batch) for batch in dc.rolling(size)]
        assert len(result) == 0

        return True

    def test_data_collection_rolling_inter(self):
        """
        target: test rolling() API for DataCollection 
        method: create an iter data collection and rolling it
        expected: return rolling successfully
        """
        data_size = 6
        data = iter(range(data_size))
        dc = towhee.dc(data)
        size = 3
        result = [list(batch) for batch in dc.rolling(size, drop_tail=False)]
        assert len(result) == data_size

        return True
    
    def test_data_collection_flatten(self):
        """
        target: test flatten() API for DataCollection 
        method: create a data collection and flatten it
        expected: return flatten successfully
        """
        data = 10
        dc = DataCollection.range(data)
        size = 3
        res = dc.batch(size)
        result = res.flatten().to_list()
        assert len(result) == data

        return True

    def test_data_collection_flatten_large_size(self):
        """
        target: test flatten() API for DataCollection 
        method: create a data collection and flatten it with large size
        expected: return flatten successfully
        """
        data = 10000000
        dc = DataCollection.range(data)
        size = 1000000
        res = dc.batch(size)
        result = res.flatten().to_list()
        assert len(result) == data

        return True
    
    def test_data_collection_flatten_size_empty(self):
        """
        target: test flatten() API for DataCollection 
        method: create a data collection and flatten it with empty size
        expected: return flatten successfully
        """
        data = 10
        dc = DataCollection.range(data)
        size = []
        res = dc.batch(size)
        result = res.flatten().to_list()
        assert len(result) == data

        return True

    def test_data_collection_flatten_inter(self):
        """
        target: test flatten() API for DataCollection
        method: create an iter data collection and flatten it
        expected: return flatten successfully
        """
        data_size = 6
        data = iter(range(data_size))
        dc = towhee.dc(data)
        size = 3
        res = dc.batch(size)
        result = res.flatten().to_list()

        assert len(result) == data_size

        return True
